import os
import math

def split_xml_raw_chunks(input_file, output_dir, lines_per_file=1000):
    """
    Splits a large XML file into smaller raw chunk files, preserving original line endings.
    This version trusts the source file's formatting and does not alter the lines.

    Args:
        input_file (str): Path to the large XML file.
        output_dir (str): Directory to save the smaller XML files.
        lines_per_file (int): Maximum number of content lines per smaller file.
    """
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
        print(f"נוצרה תיקייה: '{output_dir}'")

    try:
        with open(input_file, 'r', encoding='utf-8') as f_in:
            all_lines = f_in.readlines()

        # --- שלב 1: איתור חלקי הקובץ ---
        header_lines = []
        root_element_open_tag_line = ''
        root_element_close_tag_line = ''
        content_start_index = 0
        content_end_index = len(all_lines)

        # איתור תגית השורש הפותחת
        for i, line in enumerate(all_lines):
            stripped_line = line.strip()
            if stripped_line and stripped_line.startswith('<') and not stripped_line.startswith('<?xml'):
                root_element_name = stripped_line.split()[0].lstrip('<').rstrip('>')
                header_lines = all_lines[:i]
                root_element_open_tag_line = all_lines[i]
                content_start_index = i + 1
                
                # איתור תגית השורש הסוגרת
                expected_close_tag = f'</{root_element_name}>'
                for j in range(len(all_lines) - 1, i, -1):
                    if all_lines[j].strip() == expected_close_tag:
                        root_element_close_tag_line = all_lines[j]
                        content_end_index = j
                        break
                break

        content_lines = all_lines[content_start_index:content_end_index]

        # --- שלב 2: חלוקה וכתיבה לקבצים ---
        num_files = math.ceil(len(content_lines) / lines_per_file)

        for i in range(num_files):
            file_count = i + 1
            output_filename = os.path.join(output_dir, f'part_{file_count}.xml')
            print(f"יוצר את הקובץ: {output_filename}")

            with open(output_filename, 'w', encoding='utf-8') as output_file:
                # קובץ ראשון מקבל את הכותרת
                if i == 0:
                    output_file.writelines(header_lines)
                    output_file.write(root_element_open_tag_line)

                # כתיבת התוכן המדויק, שורה אחר שורה
                start = i * lines_per_file
                end = start + lines_per_file
                chunk = content_lines[start:end]
                output_file.writelines(chunk)

                # קובץ אחרון מקבל את התגית הסוגרת
                if i == num_files - 1:
                    print("זוהה החלק האחרון, מוסיף תגית סוגרת.")
                    output_file.write(root_element_close_tag_line)
    
    except FileNotFoundError:
        print(f"שגיאה: הקובץ '{input_file}' לא נמצא.")
    except Exception as e:
        print(f"אירעה שגיאה: {e}")


def main():
    """
    הפונקציה הראשית שמנהלת את האינטראקציה עם המשתמש.
    """
    input_file_path = input("הזן את הנתיב המלא לקובץ ה-XML לפיצול: ")

    if not os.path.isfile(input_file_path):
        print(f"שגיאה: הקובץ '{input_file_path}' לא נמצא או אינו קובץ תקין.")
        return

    output_dir_path = input("הזן את נתיב תיקיית הפלט (הקש Enter לשמירה בנתיב הנוכחי בתיקיית 'split_xml_parts'): ")

    if not output_dir_path.strip():
        output_dir_path = 'split_xml_parts'
    
    try:
        lines_per_file = int(input("כמה שורות תוכן בכל קובץ? (ברירת מחדל: 1000): ") or "1000")
    except ValueError:
        print("קלט לא חוקי, משתמש ב-1000 שורות לקובץ.")
        lines_per_file = 1000

    print("-" * 30)
    print(f"מתחיל בפיצול הקובץ '{input_file_path}'...")
    print(f"כל חלק יכיל עד {lines_per_file} שורות תוכן.")
    print(f"הקבצים יישמרו בתיקייה: '{os.path.abspath(output_dir_path)}'")
    print("-" * 30)
    
    split_xml_raw_chunks(input_file_path, output_dir_path, lines_per_file)
    
    print("-" * 30)
    print("תהליך הפיצול הסתיים בהצלחה!")
    print("-" * 30)


if __name__ == '__main__':
    main()